/*
 * Wireless Nixie Watch Code
 *
 * Created: 12/9/2017 9:50:34 PM
 * Author : emin.vartanians
 * Note: Make sure to set the user input constants before compiling
 * Note: Make sure to set fuses per comment in SetupPorts() function
 */ 

//User Input constants
#define Batt_T 3.29						//average of 3v3 regulator output between vbatt=4.2v and vbatt=3.6v
#define AccelOffset 511					//average of the the accel's 0G reading when the watch is on its back and standing up
#define AccelGain 198					//average of the accel's 1G reading based on above subtracted from the 0G reading

//Constants
#define F_CPU 1000000UL					//CPU speed based oon internal 8Mhz div 8 that actually runs code
#define Batt_i -4510					//Constant based on battery curve fit polynomial 
#define Batt_j 2.76512192234			//Constant based on battery curve fit polynomial 
#define Batt_k -0.00041452343			//Constant based on battery curve fit polynomial 



#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <stdlib.h>
#include <math.h>


void ShowDigit(uint8_t Num, uint8_t Digit,uint8_t brightness);
void DisplayHourMinute(uint8_t hours,uint8_t minutes);
void DisplayNumber(uint16_t number, uint8_t CutLeadingZeros);
void DisplayIndividualNumbers(uint8_t digit1,uint8_t digit2, uint8_t digit3, uint8_t digit4);
void SetupPorts();
void adcAccel_init();
void GetReadyForSleep();
uint16_t adc_read(uint8_t ch);
uint16_t ReadAccel(uint8_t axis);
uint8_t CheckAngle2ShowTime(uint8_t StopDisplayingTime);
void StartOneSecTimer(float HowLong);
void Start65msTimer();
void ToggleColon(uint8_t Colon);
void ChargeController();
uint8_t FlickDetector(uint8_t LastState);
int8_t DialTurner();
void SetTheTime();
void SetBrightness();
void SetAngle();
void SetHowLongToShowTime();
void SetTheDate();
void TwentyFourOrTwelve();
uint8_t Lottery(uint8_t LeftNumIn, uint8_t RightNumIn, uint8_t LeftNumOut, uint8_t RightNumOut, uint8_t LastState);
uint8_t DaysInMonth(uint8_t Month);
uint8_t TFHScorrector();
void delay_ms(uint16_t miliseconds);
void delay_us(uint16_t Tenmicroseconds);
void Menus();
uint8_t StateDetector1(uint8_t LastState);
uint8_t ColonBatteryIndicator();
void AngleToleranceCalculator();



volatile uint8_t QuartSeconds=0, Hour=12, Minute=1, Day=1, Month=12, maxday=30;						//set time and date
uint16_t ZAngleSet = AccelOffset+ AccelGain, YAngleSet = AccelOffset, XAngleSet = AccelOffset;		//set initial setpoint as watch standing straight up			
uint8_t HowLongToShowTime = 1, TFHS=12, ShowDateFlag=1, DispLottery=0, LastMinute=0, CheckBatt=1;	
uint16_t flashtimems = 1;																			//was originally gonna have the ms we flash the tube for settable but it's just a constant
int16_t BattP = 100;																				//battery percentage (pretty much only integer)
uint8_t Count65ms=0;																				//how many 65ms intervals have passed?...once it's started that is
volatile uint8_t Mode= 1;																			//Master state indicator
uint16_t Tolerance1 = 0, Tolerance2=0, Tolerance3=0,Tolerance4=0,Tolerance5=0;						//tolerances of different configs calculated once within AngleToleranceCalculator()


ISR (TIMER2_COMPA_vect)		// Timer2 ISR "Timer Keeper"
{
	QuartSeconds= QuartSeconds+1;			//increment time by 0.25 second
	if (QuartSeconds == 240){
		QuartSeconds = 0;
		Minute += 1;
		CheckBatt=1;
		if(Minute == 60){
			Minute = 0;
			Hour += 1;
			if(Hour == 24){
				Hour = 0;
				Day += 1;
				if (Day == DaysInMonth(Month) + 1){
					Day = 1;
					Month += 1;
					if (Month == 13){
						Month = 1;
					}
				}
			}
		}
	}

	
   	//if you were just asleep (mode 1) then the 250ms has expired and you should check angles (mode 2).
	//otherwise continue along.
	if (Mode == 1) {
		Mode=2;
	}

}
ISR (TIMER1_COMPA_vect)		// Timer1 ISR shortTimer (seconds)"
{
	//if Mode == 3 then it means we've been showing the time and the timer has expired. go to lottery mode (6)
	switch (Mode){
		case 3:
			if(BattP>10){Mode=6;} else{Mode=1;}		//if mode is 3 then we just finished showing the time go to lottery mode (6), unless the battery is <10% then go to sleep (1)
			break;
		case 4:
			Mode=1;				//if mode is 4 then we timed out on the menu, go back to sleep (1)
			break;	
		case 7:
			Mode=1;				//if mode is 7 then we just finished date, go back to sleep (1)			
			break;		
	}
	
}
ISR (TIMER0_OVF_vect)		// Timer1 ISR "65ms Counter"
{
	//this counter overflows every 256 counts of a the clk/256 timer. thats xFFFF microseconds which = 65,535us which rounds to 65ms.
	//count up 65ms's
	Count65ms++;
}

int main(void)
{
	SetupPorts();	//set the outputs and inputs and stuff
	AngleToleranceCalculator();		//populate the accelerometer angles matrix
	
	uint8_t StopDisplayingTime=0, LastState=0, RightAngle=0;
	
    while (1) 
    {	
		//what mode are we in?
		switch (Mode){			
			
			//sleep, don't have to show time, don't have to read angles, just sleep
			case 1:
				GetReadyForSleep();									//turn off the accel and ADC units
				sleep_cpu();
				break;
			
			//quarter second has passed, you should read the voltages and assess if we should show the time
			//If the angles are correct then change mode to 3 and show the time, otherwise change to 1 and sleep
			case 2:
				
				RightAngle = 0;		
				adcAccel_init();								//turn on the accel and ADC units	
				RightAngle=CheckAngle2ShowTime(StopDisplayingTime);				//check the ADCs and compare against set angles
				//if the angle is correct then we're gonna show time, otherwise we'll go back to sleep. but the battery must be above 5%
				if (RightAngle == 1 && StopDisplayingTime == 0 && BattP>5){						
					Mode = 3;
					Start65msTimer();
					StartOneSecTimer(HowLongToShowTime);			//start a timer that times how long the lights are on
					StopDisplayingTime = 1;						//Latch the StopDisplayTime variable so that if the shorttimer expires and you're still holding the angle you don't display again
					LastState=1;								//restart the statemachine for detecting a flick
					PORTD |= (1<<PORTD4);						//turn on boost converter
					CheckBatt=1;								//turn the checkbatt on so 250ms after you sleep you reassess if you need charging or not. You're about to show stuff on tubes and can't trust the colon not to have messed charging up
					_delay_ms(15);								//5ms just to have output kick on, 5ms to reach max, 5ms to settle at 180v
					
				//if the angle is right but the latch is on it means the watch was held at the right angle past the display time, turn it off
				} else if (RightAngle == 1 && StopDisplayingTime == 1) {
					Mode = 1;
					PORTD &= ~(1<<PORTD4);						//turn off boost converter
				//if angle is wrong at all then turn off the latch
				} else {
					Mode = 1;									//Angles were wrong, go back to sleep
					StopDisplayingTime = 0;						//if the watch angle moves out of range then unlatch the stop display so we can show it again when the angle comes back in range
					PORTD &= ~(1<<PORTD4);						//turn off boost converter
				}
				break;
				

			//Show the time. The only way out of this is if the ShortTimer changes the Mode
			case 3:
				//Display the time				
				if(ColonBatteryIndicator()<4){								//run the battery indicator for blinky stuff, if we're below 10% (==4) then just blink and go back to sleep
					DisplayHourMinute(TFHScorrector(), Minute);			//Display the time
					LastState=StateDetector1(LastState);			//take in the data and see if we've seen a double flick yet
					
				}
				break;
			
			//This is we've gone into the settings menu
			case 4:
				Menus();
				break;

			case 6:
			//lottery for a second then show time
				LastState=Lottery(TFHScorrector(), Minute, Month, Day, LastState);			//do lottery bit while still looking for a flick (last state stuff)
				if (Mode != 4){						//if Mode is not 4 then you are not in the menu system so you can go to Date, otherwise, don't mess things up because you're going to jump to menus
					ToggleColon(0);					//turn off the colon
					Mode=7;							//go to date mode
					StartOneSecTimer(1);				//show date for a second
				}
				break;
			case 7:
			//show date
				DisplayHourMinute(Month, Day);
				LastState=StateDetector1(LastState);			//take in the data and see if we've seen a double flick yet
				break;					
		}			
	}
}

//menus and menu items
void Menus(){
	
	uint8_t MenuItem=1;
	uint8_t DialFlag=1;
	uint8_t LastState = 3;							//only detecting one flick so start the stat machine in the middle state
	int8_t DialResult=0;
	uint8_t SelectionFlag=0, tempx=0, tempy=0, temp=0, adcreadflag=0, Dialflag2=0, countdownflag=0, TempMo=1, DayMoToggle=0;
	flashtimems=1;		//reset us back to nominal
	
	Start65msTimer();
	
	while(Mode == 4 ){
		

		//arbitrary 50 iterations to only do this dialturner stuff once every 50 iterations of the loop, lets downsample.
		if (Count65ms%10==0){
			if(Dialflag2){	
				DialResult = DialTurner();					//Launch DialTurner to see if you're turning left or right
				if(DialFlag){								//if you've gone back to 0 then you're free to start adding
					MenuItem = MenuItem + DialResult;		//add the result of dialturner to the menu items
				}
				if(MenuItem > 5){MenuItem=1;}				//clamp to max menu limit
				if(MenuItem < 1){MenuItem=5;}				//clamp to max menu limit										
				if (DialResult != 0){						
					StartOneSecTimer(10);						//if the dial was moved then reset the timer
					DialFlag=0;								//if the dial was moved to 1 or -1 then turn it off
					Count65ms=0;TCNT0 = 0;					//reset the 65ms timer and its counter
				}else{
					DialFlag=1;								//if the dial was moved back to 0 then re-enable it
				}
				Dialflag2=0; 
			}
		}else {
			Dialflag2=1;
		}
		
		
		//take in the data and see if we've seen a single flick yet
		LastState = FlickDetector(LastState);
		//if we've seen a single flick then we're going to set selection flag, if not, do nothing
		if( LastState == 5){
			SelectionFlag =1;
			//reset the short timer and set to 10 seconds
			StartOneSecTimer(10);			//start a timer that times how long you're in Menus before poping out
			LastState=3;
		}
		
		
		switch(MenuItem){
			//set the time
			case 1:
				ToggleColon(1);
				//if selectionflag is 1 then go into set the time but if its 0 then blink the time
				if(SelectionFlag==1){

					SetTheTime();
					SelectionFlag=0;
				} else{
					//since i counts to 100, set up a 50% duty cycle signal to either show the time or blank out the tubes.
					if(Count65ms%12 < 6){
						DisplayHourMinute(TFHScorrector(), Minute);
					} else{
						DisplayIndividualNumbers(10, 10, 10, 10);
					}	
				}
				break;
			//set 12 vs 24
			case 2:
				ToggleColon(0);
				if(SelectionFlag==1){
					TwentyFourOrTwelve();
					SelectionFlag=0;
				} else{
					//since i counts to 100, set up a 50% duty cycle signal to either show the time or blank out the tubes.

					if(Count65ms%16 > 7){
						DisplayIndividualNumbers(1, 2, 10, 10);
						} else{
						DisplayIndividualNumbers(10, 10, 2, 4);
					}
				}
				//12 or 24, flash those two back and forth
				break;
			//set angle at which we display time
			case 3:
				ToggleColon(0);
				if(SelectionFlag==1){
					SetAngle();
					SelectionFlag=0;
					LastState = 2;			//usually the angle set by user is pretty close to horizontal which means as soon as you get out of SetAngle() the flick detector sees a horizontal and goes to state=4 so when the user lifts the watch up again to vertical the detector goes to state =5 and we enter into SetAngle again. This bump back to state =2 is supposed to prevent that by requiring the user to "get out" of the horizontal zone before a flick is valid again.
				} else{
					DisplayIndividualNumbers(10,tempy, tempx, 10);
					if(Count65ms%2 == 0){
						if(adcreadflag==1){
							temp = 90* (adc_read(2)-AccelOffset)/(AccelGain);
							if(temp>90){temp=90;}else if(temp<0){temp=0;}
							tempy=temp/10;
							tempx= temp-tempy*10;
							adcreadflag=0;
						} 
					}else {
						adcreadflag=1;
					}
				}
				//Angle, show the two angles next to each other
				break;
/*
			//how long are we showing time
			case 4:
				ToggleColon(0);
				if(SelectionFlag==1){
					SetHowLongToShowTime();
					SelectionFlag=0;
				} else{
					ShowDigit(num1,1,flashtimems); ShowDigit(num2,2,flashtimems); ShowDigit(10,3,flashtimems); ShowDigit(countdownDisp,4,flashtimems);
					
					
					
					if(Count65ms%15 == 0){
						if(countdownflag==1){
							countdownDisp--;
							if(countdownDisp==0){num1=10;num2=10;}
							if(countdownDisp==255){countdownDisp=HowLongToShowTime;num1=1;num2=2;}
							countdownflag=0;
						}
					}else {
						countdownflag=1;
					}
				}
				
				break;
*/				
			//Show battery percentage by going down from 100 to the real BattP
			case 4:
				ToggleColon(0);
				
				if (Count65ms<20){
					DisplayNumber(100,1);
				}else if(Count65ms<30){
					DisplayNumber(100-((Count65ms%10)*(100-BattP)/10),1);
				}else{
					DisplayNumber(BattP,1);
				}
				
				SelectionFlag=0;
				break;				
			//Set the date
			case 5:
				ToggleColon(0);
				//if selectionflag is 1 then go into set the time but if its 0 then blink the time
				if(SelectionFlag==1){
					SetTheDate();
					SelectionFlag=0;
					DayMoToggle=0;
				} else{
					//since i counts to 100, set up a 50% duty cycle signal to either show the time or blank out the tubes.
					if(DayMoToggle==0){DisplayHourMinute(TempMo, 100);}else if(DayMoToggle==1){DisplayHourMinute(100, DaysInMonth(TempMo));}
					
						
					if(Count65ms%5 == 0){
						if(countdownflag==1){
							if(DayMoToggle==0){
								DayMoToggle=1;
							}else if(DayMoToggle==1){
								DayMoToggle=0;
								TempMo++;
								if(TempMo==13){TempMo=1;}
							}
							countdownflag=0;
						}
					}else {
						countdownflag=1;
					}		
				}
				break;
/*				
				//set Brightness
				case 8:
				ToggleColon(1);
				if(SelectionFlag==1){
					SetBrightness();
					SelectionFlag=0;
					} else{
					hTens = Hour / 10;
					hOnes = Hour - (hTens*10);
					mTens = Minute / 10;
					mOnes = Minute - (mTens*10);
					ShowDigit(hTens,1,bright); ShowDigit(hOnes,2,bright); ShowDigit(mTens,3,bright); ShowDigit(mOnes,4,bright);
				
					if(Count65ms%5 == 0){
						if(flashtimeflag==1){
							if(fadedr==1){
								bright++;
								if (bright>4){fadedr=0;bright=3;}
								} else {
								bright--;
								if (bright==0){fadedr=1;bright=2;}
							}
							flashtimeflag=0;
						}
						}else {
						flashtimeflag=1;
					}
				}
				//Brightness, show time and dim up and down
				break;				
				//Test
				case 8:
					if(SelectionFlag==1){test();} else{DisplayNumber(5);}
				break;
*/									
		}
		
				
	}
	
}
void SetTheTime(){

	int8_t tempHour=Hour;
	uint8_t tempMinute=Minute;
	uint8_t LastState = 3;							//only detecting one flick so start the stat machine in the middle state
	int8_t DialResult=0;
	uint8_t SelectionFlag=0,CounterFlag=0;
	Start65msTimer();
	
	//set the hour	
	while((Mode==4) & (SelectionFlag==0)){
		DisplayHourMinute(tempHour,100);	
		
		//arbitrary 100 iterations to only do this dialturner stuff once every 50 iterations of the loop, lets low pass filter this
		if (Count65ms%6==0){
			if(CounterFlag==1){	
				DialResult = DialTurner();					//Launch DialTurner to see if you're turning left or right
				tempHour = tempHour + DialResult;					//add the result of dialturner to the hour items
				if(tempHour == 24){tempHour=0;}						//clamp to max hour limit
				if(tempHour == -1){tempHour=23;}						//clamp to max hour limit									
				if (DialResult != 0){
					StartOneSecTimer(10);					//if the dial was moved then reset the timer
				}
				CounterFlag=0;
			}
		} else {
			CounterFlag=1;										//if we're not at limit, increment counter
		}
		
		
		//take in the data and see if we've seen a single flick yet
		LastState = FlickDetector(LastState);
		//if we've seen a single flick then we're going to set selection flag, if not, do nothing
		if( LastState == 5){
			SelectionFlag =1;
			Hour=tempHour;
		}	
	}
	StartOneSecTimer(10);			//start a timer that times how long you're in Menus before poping out
	LastState=3;
	SelectionFlag=0;
	CounterFlag=0;
	//set the minutes now
	while((Mode==4) & (SelectionFlag==0)){
		DisplayHourMinute(100,tempMinute);
			
		//arbitrary 100 iterations to only do this dialturner stuff once every 50 iterations of the loop, lets low pass filter this
		if (Count65ms%4==0){
			if(CounterFlag==1){	
				DialResult = DialTurner();					//Launch DialTurner to see if you're turning left or right
				tempMinute = tempMinute + DialResult;					//add the result of dialturner to the Minute items
				if(tempMinute == 60){tempMinute = 0;}						//clamp to max Minute limit
				if(tempMinute == 0xFF){tempMinute = 59;}						//clamp to max Minute limit
				if (DialResult != 0){
					StartOneSecTimer(10);					//if the dial was moved then reset the timer
				}
				CounterFlag=0;
			}
		} else {
			CounterFlag=1;										//if we're not at limit, increment counter
		}
				
		//take in the data and see if we've seen a single flick yet
		LastState = FlickDetector(LastState);
		//if we've seen a single flick then we're going to set selection flag, if not, do nothing
		if( LastState == 5){
			SelectionFlag =1;
			Minute=tempMinute;
			//once everything has been selected, set the seconds to 0 as well to reset all counts
			QuartSeconds=0;
		}
	}
	

return;
}
void TwentyFourOrTwelve(){

	uint8_t tempTFHS=TFHS;
	uint8_t LastState = 3;							//only detecting one flick so start the stat machine in the middle state
	int8_t DialResult=0;
	uint8_t SelectionFlag=0, CounterFlag=0;
	Start65msTimer();
	
	//set the hour
	while((Mode==4) & (SelectionFlag==0)){
		if(tempTFHS==12){
			DisplayIndividualNumbers(1, 2,10,10);
		} else if (tempTFHS==24){
			DisplayIndividualNumbers(10, 10,2,4);
		}
		
		//arbitrary 100 iterations to only do this dialturner stuff once every 50 iterations of the loop, lets low pass filter this
		if (Count65ms%4==0){
			if(CounterFlag==1){	
				DialResult = DialTurner();					//Launch DialTurner to see if you're turning left or right
				tempTFHS = tempTFHS + DialResult*12;					//add the result of dialturner to the hour items
				if(tempTFHS > 24){tempTFHS=24;}						//clamp to max hour limit
				if(tempTFHS < 12){tempTFHS=12;}						//clamp to max hour limit										
				if (DialResult != 0){
					StartOneSecTimer(10);					//if the dial was moved then reset the timer
				}
				CounterFlag=0;
			}
		} else {
			CounterFlag=1;										//if we're not at limit, increment counter
		}
		
		
		//take in the data and see if we've seen a single flick yet
		LastState = FlickDetector(LastState);
		//if we've seen a single flick then we're going to set selection flag, if not, do nothing
		if(LastState== 5){
			SelectionFlag = 1;
			TFHS=tempTFHS;
		}
	}
	return;
}
void SetBrightness(){

	uint8_t hTens = Hour / 10;
	uint8_t hOnes = Hour - (hTens*10);
	uint8_t mTens = Minute / 10;
	uint8_t mOnes = Minute - (mTens*10);
	uint8_t LastState = 3;							//only detecting one flick so start the stat machine in the middle state
	uint8_t tempFlashtimems=flashtimems;
	int8_t DialResult=0;
	uint8_t SelectionFlag=0, CounterFlag=0;
	Start65msTimer();
	
	//set the hour
	while((Mode==4) & (SelectionFlag==0)){

		ShowDigit(hTens,1,tempFlashtimems); ShowDigit(hOnes,2,tempFlashtimems); ShowDigit(mTens,3,tempFlashtimems); ShowDigit(mOnes,4,tempFlashtimems);
		
		//arbitrary 100 iterations to only do this dialturner stuff once every 50 iterations of the loop, lets low pass filter this
		if (Count65ms%6==0){
			if(CounterFlag==1){
				DialResult = DialTurner();					//Launch DialTurner to see if you're turning left or right
				tempFlashtimems = tempFlashtimems + DialResult;					//add the result of dialturner to the hour items
				if(tempFlashtimems > 4){tempFlashtimems=4;}						//clamp to max hour limit
				if(tempFlashtimems < 1){tempFlashtimems=1;}						//clamp to max hour limit
				if (DialResult != 0){
					StartOneSecTimer(10);					//if the dial was moved then reset the timer
				}
				CounterFlag=0;
			}
		} else {
			CounterFlag=1;										//if we're not at limit, increment counter
		}
		
		
		//take in the data and see if we've seen a single flick yet
		LastState = FlickDetector(LastState);
		//if we've seen a single flick then we're going to set selection flag, if not, do nothing
		if(LastState== 5){
			SelectionFlag = 1;
			flashtimems=tempFlashtimems;
		}
	}
	return;
}
void SetAngle(){
	
	uint16_t localzcount = 0, localycount = 0, localxcount = 0;
	uint8_t countdownflag=0, adcreadflag=0, countdown1=5, countdown2=5, i=0;
	Start65msTimer();
	StartOneSecTimer(20);
	
	//wait 5 seconds for user to get in position
	while((Mode==4) & (countdown1>0)){
		ShowDigit(countdown1,1,flashtimems);ShowDigit(10,2,flashtimems);ShowDigit(10,3,flashtimems); ShowDigit(countdown2,4,flashtimems);

		if(Count65ms%15 == 0){						//count a rough second
			if(countdownflag==1){
				countdown1--;
				countdownflag=0;
			}
		}else {
		countdownflag=1;
		}		
	}
	

	Start65msTimer();//reset timer
	while((Mode==4) & (countdown2>0)){	
		ShowDigit(countdown1,1,flashtimems);ShowDigit(10,2,flashtimems);ShowDigit(10,3,flashtimems); ShowDigit(countdown2,4,flashtimems);


		if(Count65ms%2 == 0){						//do this every 130ms
			if(adcreadflag==1){
				localzcount = localzcount + adc_read(2);
				localycount = localycount + adc_read(3);
				localxcount = localxcount + adc_read(4);
				i++;
				adcreadflag=0;
			}
		}else {
			adcreadflag=1;
		}
		
		
		if(Count65ms%15 == 0){						//count a rough second
			if(countdownflag==1){
				countdown2--;
				countdownflag=0;
			}
		}else {
			countdownflag=1;
		}		
		
	}
	
	ZAngleSet = localzcount/i;
	YAngleSet = localycount/i;
	XAngleSet = localxcount/i;
	
	return;
}
void SetTheDate(){

	uint8_t tempDay=Day;
	int8_t tempMonth=Month;
	int8_t maxDays=DaysInMonth(tempMonth);
	uint8_t LastState = 3;							//only detecting one flick so start the stat machine in the middle state
	int8_t DialResult=0;
	uint8_t SelectionFlag=0,CounterFlag=0;

	Start65msTimer();

	//set the month
	while((Mode==4) & (SelectionFlag==0)){
//		if (tempMonth==0){
//			DisplayHourMinute(100,100);
//		} else {
			DisplayHourMinute(tempMonth,100);
//		}
		
		
		//arbitrary 100 iterations to only do this dialturner stuff once every 50 iterations of the loop, lets low pass filter this
		if (Count65ms%6==0){
			if(CounterFlag==1){
				DialResult = DialTurner();					//Launch DialTurner to see if you're turning left or right
				tempMonth = tempMonth + DialResult;					//add the result of dialturner to the hour items
				if(tempMonth == 13){tempMonth=1;}						//clamp to max hour limit
				if(tempMonth == 0){tempMonth=12;}						//clamp to max hour limit
				if (DialResult != 0){
					StartOneSecTimer(10);					//if the dial was moved then reset the timer
				}
				CounterFlag=0;
			}
		} else {
			CounterFlag=1;										//if we're not at limit, increment counter
		}
	
	
		//take in the data and see if we've seen a single flick yet
		LastState = FlickDetector(LastState);
		//if we've seen a single flick then we're going to set selection flag, if not, do nothing
		if( LastState == 5){
			SelectionFlag =1;
			Month=tempMonth;
		}
	}
	StartOneSecTimer(10);			//start a timer that times how long you're in Menus before poping out
	LastState=3;
	SelectionFlag=0;
	CounterFlag=0;
	maxDays=DaysInMonth(tempMonth);
	//set the minutes now
	while((Mode==4) & (SelectionFlag==0) & (tempMonth!=0)){
		DisplayHourMinute(100,tempDay);
	
		//arbitrary 100 iterations to only do this dialturner stuff once every 50 iterations of the loop, lets low pass filter this
		if (Count65ms%4==0){
			if(CounterFlag==1){
				DialResult = DialTurner();					//Launch DialTurner to see if you're turning left or right
				tempDay = tempDay + DialResult;					//add the result of dialturner to the Minute items
				if(tempDay == maxDays+1){tempDay = 1;}						//clamp to max Minute limit
				if(tempDay == 0){tempDay = maxDays;}						//clamp to max Minute limit
				if (DialResult != 0){
					StartOneSecTimer(10);					//if the dial was moved then reset the timer
				}
				CounterFlag=0;
			}
			} else {
			CounterFlag=1;										//if we're not at limit, increment counter
		}
	
		//take in the data and see if we've seen a single flick yet
		LastState = FlickDetector(LastState);
		//if we've seen a single flick then we're going to set selection flag, if not, do nothing
		if( LastState == 5){
			SelectionFlag =1;
			Day=tempDay;
		}
	}
	

	return;
}

//display functions
void DisplayNumber(uint16_t number, uint8_t CutLeadingZeros){
	//All this math takes 4.5ms to do
	int Thousands,Hundreds,Tens, Ones;
	Thousands = number / 1000;
	Hundreds = (number - (Thousands * 1000)) / 100;
	Tens = (number - (Thousands * 1000) - (Hundreds * 100)) / 10;
	Ones = number - (Thousands * 1000) - (Hundreds * 100) - (Tens * 10);
	
	if((CutLeadingZeros) & (number<10)){Tens=10;}
	if((CutLeadingZeros) & (number<100)){Hundreds=10;}
	if((CutLeadingZeros) & (number<1000)){Thousands=10;}	
	if(number>9999){number = 0xFFFF;}
	if (number==0xFFFF){Thousands=10;Hundreds=10;Tens=10;Ones=10;}		//if you input 0xFFFF then blank out the numbers

	ShowDigit(Thousands, 1, flashtimems);ShowDigit(Hundreds,2, flashtimems);ShowDigit(Tens, 3, flashtimems);ShowDigit(Ones, 4, flashtimems);	 //show the number
}
void DisplayHourMinute(uint8_t hours,uint8_t minutes){
	uint8_t hTens, hOnes, mTens, mOnes;
	
	if(hours<100){
		hTens = hours / 10;
		hOnes = hours - (hTens*10);
	} else {
		hTens=10;
		hOnes=10;
	}
	
	if(minutes<100){
		mTens = minutes / 10;
		mOnes = minutes - (mTens*10);
	} else{
		mTens=10;
		mOnes=10;
	}
	
ShowDigit(hTens,1, flashtimems); ShowDigit(hOnes, 2, flashtimems); ShowDigit(mTens, 3, flashtimems); ShowDigit(mOnes, 4, flashtimems); //show the number
}
void DisplayIndividualNumbers(uint8_t digit1,uint8_t digit2, uint8_t digit3, uint8_t digit4){
	ShowDigit(digit1,1, flashtimems); ShowDigit(digit2, 2, flashtimems); ShowDigit(digit3, 3, flashtimems); ShowDigit(digit4, 4, flashtimems); //show the number
	
}
void ShowDigit(uint8_t Num, uint8_t Digit,uint8_t brightness)
{
	//fire the tube and column / row manner		
	switch (Num){
		case 0:
		PORTE |= (1<<PORTE1);break;
		case 1:
		PORTE |= (1<<PORTE2);break;
		case 2:
		PORTC |= (1<<PORTC0);break;
		case 3:
		PORTD |= (1<<PORTD2);break;
		case 4:
		PORTD |= (1<<PORTD1);break;
		case 5:
		PORTD |= (1<<PORTD5);break;
		case 6:
		PORTB |= (1<<PORTB2);break;
		case 7:
		PORTD |= (1<<PORTD7);break;
		case 8:
		PORTB |= (1<<PORTB0);break;
		case 9:
		PORTE |= (1<<PORTE0);break;
	}
	
	
	//turn on the digit only if the Num is good, otherwise just fake the flashtime and leave
	if((Num>=0) & (Num<10)){
		switch (Digit){
			case 1:
			PORTE |= (1<<PORTE3);delay_ms(brightness);PORTE &= ~(1<<PORTE3);break;
			case 2:
			PORTC |= (1<<PORTC1);delay_ms(brightness);PORTC &= ~(1<<PORTC1);break;
			case 3:
			PORTD |= (1<<PORTD6);delay_ms(brightness);PORTD &= ~(1<<PORTD6);break;
			case 4:
			PORTB |= (1<<PORTB1);delay_ms(brightness);PORTB &= ~(1<<PORTB1);break;
		} 
	} else{ 
		delay_ms(brightness);
	}
	
	_delay_us(50);			//wait tiny amount to allow for charge to drain away from the current tube/numeral and let the regulator voltage come back to 180v
	
	//turn on the number
	switch (Num){
		case 0:
		PORTE &= ~(1<<PORTE1);break;
		case 1:
		PORTE &= ~(1<<PORTE2);break;
		case 2:
		PORTC &= ~(1<<PORTC0);break;
		case 3:
		PORTD &= ~(1<<PORTD2);break;
		case 4:
		PORTD &= ~(1<<PORTD1);break;
		case 5:
		PORTD &= ~(1<<PORTD5);break;
		case 6:
		PORTB &= ~(1<<PORTB2);break;
		case 7:
		PORTD &= ~(1<<PORTD7);break;
		case 8:
		PORTB &= ~(1<<PORTB0);break;
		case 9:
		PORTE &= ~(1<<PORTE0);break;
	}

	
	
	return;

}

//Setup and housekeeping functions
void SetupPorts(){
	
	//NEVER SET FUSES TO EXTERNAL OSCILATOR
	//these are for internal 8Mhz with 8div so 1Mhz.
	//This code is written to run at 1Mhz and merely count time off the exteranl clock (ASSR register below)
	//lfuse:w:0x42:m -U hfuse:w:0xd7:m -U efuse:w:0xf9:m
	
	//Set up the inputs and outputs
	DDRB = 0xAF; DDRC = 0x03; DDRD = 0xFF; DDRE = 0xFF;
	
	
	//should I hold this as an input (Hi-Z) or output high (3.3)? somehow the Hi-Z state might be less current?
	PORTB |= (1<<PORTB7);		//turn off the battsense (set to 3.3v such that pfet is off and no current is flowing thru v-divider)

//Initialize the counter to count up to 1 second
	ASSR |= (1<<EXCLK);							//enable external clock mode for TCXO instead of crystal
	ASSR |= (1<<AS2);							//TIMER IN ASYNC MODE
	TCCR2A = (1<<WGM21);						//set mode to CTC
	//TCCR2B	= (1<<CS22) | (1<<CS21)| (1<<CS20);		//Fake one just force long sleep. Now we have 32768/256 = 32 ticks per sec
	TCCR2B = (1<<CS22) | (1<<CS21);				//prescaler 32768/256 =128 per second
	TIMSK2 = (1<<OCIE2A);						//unmask the timer0_compA for interrupt service
	//OCR2A =	255;									//fake one just to force long sleep. Now we have 255 ticks/32 ticks per sec = 8 sec 
	OCR2A = 31;									//set the counter value, (128-1)=1 second, (64-1)=twice a second
	sei();										//enable global interrupt
	_delay_ms(500);								//give the timer circuit enough time to get itself going before we might go to sleep
	

//	PORTD |= (1<<PORTD4);		//boost converter on
//	PORTD |= (1<<PORTD3);		//charge low so it charges
	
//set sleep mode register	
	SMCR =	(1<<SM2) | (1<<SM1) | (1<<SM0);		//set sleep mode as extended standby 
	sleep_enable();

//disable digital inputs connected to ADC input pins
//since ADC channels 2, 3, 4, and 5 are used we build 0b0011110=0d30
	DIDR0 = 30; 
	
//set PRR register to turn off all unused peripherals
//7=TWI, 6=TC2, 5=TC0, 3=TC1, 2=SPI, 1=USART, 0=ADC
//Since we only need TC2 and SPI to always work, and ADC is set but ADC init we have this:
	PRR = 186;	//	0d186 = 0b10111010;		
	
}
void adcAccel_init(){
//turn on the adc in the power reduction register
	PRR &= ~(1<<PRADC); 

//this means turn on the ADC and set the clock division to 32 to get 31khz
//adc_clock = 125khz is fine for Accel but too fast for batt voltage
//884 at 125khz vs 953 at 15khz vs 954 at 7.8khz vs 937 at 62khz vs 950 at 31khz
//seems like anything above 31khz is too fast, will have to try puting a cap on R24 and see how that changes these numbers
//installed 4.7uf cap: 960 at 31khz, 960 at 62khz,  960 at 125khz, 960 at 250khz, 960 at 500khz
	//ADCSRA |= (1<<ADEN) | (1<<ADPS0) | (1<<ADPS1);			//divide 1M by 8 so 125khz.
	ADCSRA |= (1<<ADEN) | (1<<ADPS0) | (1<<ADPS2);		//divide 1M by 32 so ~62khz
	
//turn on the accel	
	PORTD |= (1<<PORTD0);									//turn on accel
	_delay_ms(15);											//let output settle
}
void GetReadyForSleep(){
//turn off the adc
	ADCSRA =0;
//turn off the adc module, timer0 (65ms timer) module, and timer1 (short timer) module in the power reduction register
	PRR |= (1<<PRADC) | (1<<PRTIM1) | (1<<PRTIM0);
//turn off the accel	
	PORTD &= ~(1<<PORTD0);
//turn off the boost converter	
	PORTD &= ~(1<<PORTD4);
//turn off the Battsense valve
	PORTB |= (1<<PORTB7);
//make sure we go to sleep going back to 1ms flashtime and not randmly latched to the higher value
	flashtimems=1;
}
uint16_t adc_read(uint8_t ch)
{
	
	uint16_t total=0;
	
	// select the corresponding channel 0~7
	// ANDing with '7' will always keep the value
	// of 'ch' between 0 and 7
	ch &= 0b00001111;  // AND operation with 15. takes 0us
	ADMUX = ch + 64;	//64 is 0100 0000 which means NOT left aligning the data and choosing "AVcc with external cap on Aref" as Vref
//	ADMUX = (ADMUX & 0xF0)|ch;     // clears the bottom 4 bits before ORing, takes likes 200us!
//	ADCSRA = (1<<ADEN);
//	ADCSRA = 128;
	
	
//	count = TCNT1;
//	uint8_t i=0;
//	for(i=0;i<1;i++)
//	{
			// set ADSC bit to get the *next* conversion started
		ADCSRA |= (1<<ADSC);				//takes 8 ticks of adc clock
		//ADCSRA |= (1<<ADSC);		//takes 170us to write!
			// wait for conversion to complete
			// ADSC becomes '0' again
			// till then, run loop continuously
		
		while(ADCSRA & (1<<ADSC));		//takes ~1000us
		
		
//		if (ADC > total){
			total = ADC;
//		}	//add the samples for a final averaging, takes 150us, 5 ticks of adc clock
		
	
//	}
//	count = TCNT1- count ;
	//total = total/3;			//divide by 12 samples to get average
	
//	ADCSRA &= ~(1<<ADEN);

	return (total);	
}		
void ToggleColon(uint8_t Colon){
	
	//set pin as output. If colon==1 then drive pin low to turn on colon, if colon==0 then drive pin high to turn off colon
	//effects of this function will reset once the micro goes to sleep and comes back into mode2 where it executes ChargeController()
	DDRD |= (1<<DDRD3);
	if(Colon){
		PORTD &= ~(1<<PORTD3);
	} else{
		PORTD |= (1<<PORTD3);
	}
}
void AngleToleranceCalculator(){
//This function sets the tolerance of the 4 places we evaluate the angle.
//This only runs once at startup...


Tolerance1 = ((float) AccelGain / 3);				//measure of 20 degrees in off of 0G position (in counts)	->used for determination of horizontalness
Tolerance2 = ((float) AccelGain * 0.707);			//measure of 45 degrees in off of 1G position (in counts)	->used for determination of verticalness
Tolerance3 = ((float) AccelGain * 50 / 200);		//measures about 15 degrees off of 0G position (in counts)	->used to see if we should turn on tubes
Tolerance4 = ((float) AccelGain * 100 / 200);		//measures about 30 degrees off of 0G position (in counts)	->used to see if we should turn on tubes
Tolerance5 = ((float) AccelGain * 80 / 200);		//measures about 25 degrees off of 0G position (in counts)	->used for dial turn
}
void ChargeController(){
	
	//970 is 90%, 960 is 86%. 986 is 97%. A bit of hysteresis
	//above battery being at 90% we shut off the charging by setting the pin as input with no pullup 
	//(if charge is already happening then the charger chip holds this pin down anyway)
	//below battery being at 86% we turn on the charging by setting the pin as output low.

	if(BattP>90){ 
		DDRD &= ~(1<<DDRD3);				//configure CHRG! pin as input
		PORTD |= (1<<PORTD3);				//engage 50k pull-up resistor
		_delay_us(10);						//wait for the voltages to settle
		if((PIND & 0x08) == 0){				//pin is staying pulled low seems like you're still charging
			PORTD &= ~(1<<PORTD3);			//Drive pin low so that you keep charging			
			DDRD |= (1<<DDRD3);				//switch pin to output
		}else{								//pin is being pulled high seems like you're done charging or you're not charging
			DDRD |= (1<<DDRD3);				//switch pin to output
			PORTD |= (1<<PORTD3);			//Drive pin high so that you stop charging
		}
	}

	if (BattP<=90){
		DDRD |= (1<<DDRD3);					//switch pin to output
		PORTD &= ~(1<<PORTD3);				//drive pin low so that you start charging
	}
}
uint8_t ColonBatteryIndicator(){
	//evaluates the battery level against an index then
	//blinks the colon based on that index to show user battery level
	
	uint8_t BatterylevelIndex=0;
	if(BattP<75){BatterylevelIndex=1;}
	if(BattP<50){BatterylevelIndex=2;}
	if(BattP<25){BatterylevelIndex=3;}
	if(BattP<10){BatterylevelIndex=4;}	
	
	int toggleflag=1;
	
	switch (BatterylevelIndex){
		case 1: if((Count65ms==3) | (Count65ms==4)){toggleflag=0;} else {toggleflag=1;} break;
		case 2: if((Count65ms==3) | (Count65ms==4) | (Count65ms==7) | (Count65ms==8)){toggleflag=0;} else {toggleflag=1;} break;	
		case 3: if((Count65ms==3) | (Count65ms==4) | (Count65ms==7) | (Count65ms==8) | (Count65ms==11) | (Count65ms==12)){toggleflag=0;} else {toggleflag=1;} break;	
		case 4: if((Count65ms==3) | (Count65ms==4) | (Count65ms==7) | (Count65ms==8) | (Count65ms==11) | (Count65ms==12) | (Count65ms==15) | (Count65ms==16)){toggleflag=0;} else {toggleflag=1;} break;
	}
	
	ToggleColon(toggleflag);	
	return BatterylevelIndex;	

}

//Timekeeping Functions
uint8_t DaysInMonth(uint8_t Month) {
	return 28 + (Month + (Month/8)) % 2 + 2 % Month + 2 * (1/Month); 
}
uint8_t TFHScorrector(){
	uint8_t HourDisp=0;
	if(TFHS==12){
		HourDisp=Hour%12;
		if((Hour==12)|(Hour==0)){
			HourDisp=12;
		}
	} else{
		HourDisp=Hour;
	}
	return HourDisp;
}
void delay_ms(uint16_t miliseconds){
	
	uint16_t i=0;
	while (i<miliseconds){
		_delay_ms(1);
		i++;
	}
	return;
}
void delay_us(uint16_t Tenmicroseconds){
	
	uint16_t i=0;
	while (i<Tenmicroseconds){
		_delay_us(10);
		i++;
	}
	return;
}
void StartOneSecTimer(float HowLong){
//starts a timer and sets the HowLong (in seconds)	
	TCNT1 = 0;															//reset the timercounter value
	PRR &= ~(1<<PRTIM1);												//turn off the power reduction for timer 1
	TCCR1B = (1<<WGM12) | (1<<CS10) | (1<<CS12);						//set mode to CTC, set prescaler to 1024 so 1Mhz/1024 = 976 per second
	TIMSK1 = (1<<OCIE1A);												//unmask the timer1_compA for interrupt service
	OCR1A = HowLong * 976;												//set the counter value, HowLong is in seconds so multiply by 976
}
void Start65msTimer(){
	//this counter overflows every 256 counts of a the clk/256 timer. thats xFFFF microseconds which = 65,535us which rounds to 65ms.
	//starts a 65ms timer
	Count65ms=0;
	TCNT0 = 0;															//reset the timercounter value
	PRR &= ~(1<<PRTIM0);												//turn off the power reduction for timer 1
	TCCR0B = (1<<CS02);													//set mode to normal mode so it overflows when it hits 0xFF. Prescaler is 256 so 1Mhz/256 = 3.9khz counting to 0xFF thats 15.25hz = 65.5ms
	TIMSK0 = (1<<TOIE0);												//unmask the timer0_overflow for ISR
}


//User Interface functions
uint8_t StateDetector1(uint8_t LastState){
	//take in the data and see if we've seen a double flick yet
	LastState = FlickDetector(LastState);
	//if we've seen a double flick then we're going to settings menu mode, if not, keep displaying the time
	if( LastState == 5){
		Mode = 4;
		//reset the short timer and set to 10 seconds
		StartOneSecTimer(10);			//start a timer that times how long you're in Menus before popping out
	}
	return LastState;
}
uint8_t FlickDetector(uint8_t LastState){
	//Generally looking for a light-down-up-down-up sequence

	uint16_t localzcount = adc_read(2);
	uint16_t localycount = adc_read(3);
	uint16_t localxcount = adc_read(4);

	
	//crude state machine used for detecting a double flick
	uint8_t state=LastState;
	switch(LastState){
		//you've just entered this place, look for vertical position
		case 0:
		if ((localxcount>AccelOffset-Tolerance1) & (localxcount<AccelOffset+Tolerance1) & (localycount>AccelOffset-Tolerance1) & (localycount<AccelOffset+Tolerance1) & (localzcount>AccelOffset+Tolerance2) & (localzcount<750)){
			state =1;
		}
		break;
		//you're vertical, look for horizontal
		case 1:
		if ((localxcount>AccelOffset-Tolerance1) & (localxcount<AccelOffset+Tolerance1) & (localycount>AccelOffset+Tolerance2) & (localycount<750) & (localzcount>AccelOffset-Tolerance1) & (localzcount<AccelOffset+Tolerance1)){
			state = 2;
		}
		break;
		//you're horizontal, look for vertical
		case 2:
		if ((localxcount>AccelOffset-Tolerance1) & (localxcount<AccelOffset+Tolerance1) & (localycount>AccelOffset-Tolerance1) & (localycount<AccelOffset+Tolerance1) & (localzcount>AccelOffset+Tolerance2) & (localzcount<750)){
			state = 3;
		}
		break;
		//you're vertical, look for horizontal
		case 3:
		if ((localxcount>AccelOffset-Tolerance1) & (localxcount<AccelOffset+Tolerance1) & (localycount>AccelOffset+Tolerance2) & (localycount<750) & (localzcount>AccelOffset-Tolerance1) & (localzcount<AccelOffset+Tolerance1)){
			state = 4;
		}
		break;
		//you're horizontal, look for vertical
		case 4:
		if ((localxcount>AccelOffset-Tolerance1) & (localxcount<AccelOffset+Tolerance1) & (localycount>AccelOffset-Tolerance1) & (localycount<AccelOffset+Tolerance1) & (localzcount>AccelOffset+Tolerance2) & (localzcount<750)){
			state = 5;
		}
		break;
	}
	
	return state;
}
uint8_t CheckAngle2ShowTime(uint8_t StopDisplayingTime){
//reads through all 3 axes of accel and if the angles are correct it returns a 1

	uint16_t localzcount = adc_read(2);
	uint16_t localycount = adc_read(3);
	uint16_t localxcount = adc_read(4);

	if (CheckBatt){
		uint16_t countb = 0;	
		PORTB &= ~(1<<PORTB7);					//turn on the valve for reading the battsense line (0v since pfet)
		countb = adc_read(5);					//read the batt voltage
		PORTB |= (1<<PORTB7);					//just read the battsense line so no need for the valve to stay open. Turn it off (3.3v since pfet
		CheckBatt=0;							//take down the flag until a second has passed
		BattP=Batt_i + (Batt_j * Batt_T * countb) + (Batt_k * Batt_T * Batt_T * countb * countb);		//formula related Battery Percentage to 3v3 rail and batt_sense count		
		ChargeController();						//see if we need to charge
	}
	uint8_t tolerance = Tolerance3;
	
	//StopDisplayingTime being 0 or 1 says if we are good to display or not. Use this for hystereis
	if (StopDisplayingTime==0){
		tolerance = Tolerance3;
	} else if (StopDisplayingTime==1){
		tolerance =Tolerance4;
	}
	uint8_t result =0;
	if ((ZAngleSet-tolerance < localzcount) & (localzcount < ZAngleSet +tolerance) & (YAngleSet-tolerance < localycount) & (localycount < YAngleSet +tolerance) & (XAngleSet-tolerance < localxcount) & (localxcount < XAngleSet +tolerance)){
		result= 1;
	} else{
		result= 0;
	}	
	return result;
}
uint8_t Lottery(uint8_t LeftNumIn, uint8_t RightNumIn, uint8_t LeftNumOut, uint8_t RightNumOut, uint8_t LastState){

	uint8_t StartNum1=LeftNumIn/10;
	uint8_t StartNum2=LeftNumIn - (10*StartNum1);
	uint8_t StartNum3=RightNumIn/10;
	uint8_t StartNum4=RightNumIn - (10*StartNum3);
	uint8_t EndNum1=LeftNumOut/10;
	uint8_t EndNum2=LeftNumOut - (10*EndNum1);
	uint8_t EndNum3=RightNumOut/10;
	uint8_t EndNum4=RightNumOut - (10*EndNum3);		
			
	int8_t Temp1=StartNum1, Temp2=StartNum2, Temp3=StartNum3, Temp4=StartNum4;
	uint8_t done1=0, done2=0, done3=0, done4=0;
	//keep it interesting, if a number is not going to move at all, add one so it has to move a bunch
	if (Temp1==EndNum1){Temp1++; if(Temp1==10){Temp1=0;}}
	if (Temp2==EndNum2){Temp2++; if(Temp2==10){Temp2=0;}}
	if (Temp3==EndNum3){Temp3++; if(Temp3==10){Temp3=0;}}
	if (Temp4==EndNum4){Temp4++; if(Temp4==10){Temp4=0;}}		
			
	uint8_t i=0;
	while(!(done1 & done2 & done3 & done4) & (LastState!=5)) {				//in the while loop until all of the numbers match the "end" state
		LastState=StateDetector1(LastState);													//take in the data and see if we've seen a double flick yet
		DisplayIndividualNumbers(Temp1, Temp2, Temp3, Temp4);
		if(i>5){
			if(Temp1!=EndNum1){Temp1++; if(Temp1==10){Temp1=0;}}else{done1=1;}
			if(Temp2!=EndNum2){Temp2++; if(Temp2==10){Temp2=0;}}else{done2=1;}
			if(Temp3!=EndNum3){Temp3++; if(Temp3==10){Temp3=0;}}else{done3=1;}
			if(Temp4!=EndNum4){Temp4++; if(Temp4==10){Temp4=0;}}else{done4=1;}
			i=0;
		}else {
			i++;
		}
	}
	return LastState;
}
int8_t DialTurner(){
	//reads through all 3 axes of accel and if the angles are correct it returns a 1
	uint16_t localzcount = adc_read(2);
//	uint16_t localycount = adc_read(3);
	uint16_t localxcount = adc_read(4);
	int8_t result =0;
	
	
	if(localzcount > AccelOffset){
		if (localxcount < AccelOffset-Tolerance5){
			result= -1;
		} 
		else if (localxcount > AccelOffset+Tolerance5){
			result= 1;
		}
	}
	return result;
}

